@@ -67,6 +67,11 @@ def git_get_rev_diff(gitdir, rev):
     return b4.git_run_command(gitdir, args)
 
 
+def git_get_commit_message(gitdir, rev):
+    args = ['log', '--format=%B', '-1', rev]
+    return b4.git_run_command(gitdir, args)
+
+
 def make_reply(reply_template, jsondata):
     body = Template(reply_template).safe_substitute(jsondata)
     # Conform to email standards
@@ -164,7 +169,15 @@ def get_all_commits(gitdir, branch, since='1.week', committer=None):
         ecode, out = git_get_rev_diff(gitdir, commit_id)
         pwhash = b4.LoreMessage.get_patchwork_hash(out)
         logger.debug('phash=%s', pwhash)
-        MY_COMMITS[pwhash] = (commit_id, subject)
+        # get all message-id or link trailers
+        ecode, out = git_get_commit_message(gitdir, commit_id)
+        matches = re.findall(r'^\s*(?:message-id|link):[ \t]+(\S+)\s*$', out, flags=re.I | re.M)
+        trackers = list()
+        if matches:
+            for tvalue in matches:
+                trackers.append(tvalue)
+
+        MY_COMMITS[pwhash] = (commit_id, subject, trackers)
 
     return MY_COMMITS
 
@@ -187,18 +200,27 @@ def auto_locate_series(gitdir, jsondata, branch, since='1.week'):
             success = False
             for pwhash, commit in commits.items():
                 if commit[1] == patch[0]:
+                    logger.debug('Matched using subject')
                     found.append(commit)
                     success = True
                     matches += 1
                     break
+                elif len(patch) > 2 and len(patch[2]) and len(commit[2]):
+                    for tracker in commit[2]:
+                        if tracker.find(patch[2]) >= 0:
+                            logger.debug('Matched using recorded message-id')
+                            found.append(commit)
+                            success = True
+                            matches += 1
+                            break
+                if success:
+                    break
+
             if not success:
                 logger.debug('  Failed to find a match for: %s', patch[0])
                 found.append((None, patch[0]))
 
-    if matches > 0:
-        return found
-
-    return None
+    return found
 
 
 def read_template(tptfile):
@@ -294,29 +316,28 @@ def generate_am_thanks(gitdir, jsondata, branch, since):
     else:
         commits = jsondata['commits']
 
-    if commits is None:
-        logger.critical('Could not match any commits for: %s', jsondata['subject'])
-        logger.critical('Not thanking for this series')
-        return None
     cidmask = config['thanks-commit-url-mask']
     if not cidmask:
         cidmask = 'commit: %s'
     slines = list()
-    counter = 1
-    partial = False
+    counter = 0
+    nomatch = 0
     padlen = len(str(len(commits)))
     for commit in commits:
+        counter += 1
         prefix = '[%s/%s] ' % (str(counter).zfill(padlen), len(commits))
         slines.append('%s%s' % (prefix, commit[1]))
         if commit[0] is None:
-            slines.append('%s(not applied)' % (' ' * len(prefix)))
-            partial = True
+            slines.append('%s(no commit info)' % (' ' * len(prefix)))
+            nomatch += 1
         else:
             slines.append('%s%s' % (' ' * len(prefix), cidmask % commit[0]))
-        counter += 1
     jsondata['summary'] = '\n'.join(slines)
-    if partial:
-        logger.critical('  WARNING: Not all patches matched for: %s', jsondata['subject'])
+    if nomatch == counter:
+        logger.critical('  WARNING: None of the patches matched for: %s', jsondata['subject'])
+        logger.critical('           Please review the resulting message')
+    elif nomatch > 0:
+        logger.critical('  WARNING: Could not match %s of %s patches in: %s', nomatch, counter, jsondata['subject'])
         logger.critical('           Please review the resulting message')
 
     msg = make_reply(thanks_template, jsondata)
@@ -343,7 +364,13 @@ def auto_thankanator(cmdargs):
         else:
             # This is a patch series
             commits = auto_locate_series(gitdir, jsondata, wantbranch, since=cmdargs.since)
-            if commits is None:
+            # Weed out series that have no matches at all
+            found = False
+            for commit in commits:
+                if commit[0] is not None:
+                    found = True
+                    break
+            if not found:
                 continue
             jsondata['commits'] = commits
         applied.append(jsondata)
@@ -450,11 +477,11 @@ def send_selected(cmdargs):
         logger.info('Nothing to do')
         sys.exit(0)
 
-    if 'all' in cmdargs.send:
+    if cmdargs.send == 'all':
         listing = tracked
     else:
         listing = list()
-        for num in cmdargs.send:
+        for num in b4.parse_int_range(cmdargs.send, upper=len(tracked)):
             try:
                 index = int(num) - 1
                 listing.append(tracked[index])
@@ -483,11 +510,11 @@ def discard_selected(cmdargs):
         logger.info('Nothing to do')
         sys.exit(0)
 
-    if 'all' in cmdargs.discard:
+    if cmdargs.discard == 'all':
         listing = tracked
     else:
         listing = list()
-        for num in cmdargs.discard:
+        for num in b4.parse_int_range(cmdargs.discard, upper=len(tracked)):
             try:
                 index = int(num) - 1
                 listing.append(tracked[index])
@@ -531,21 +558,21 @@ def get_wanted_branch(cmdargs):
     gitdir = cmdargs.gitdir
     if not cmdargs.branch:
         # Find out our current branch
-        gitargs = ['rev-parse', '--abbrev-ref', 'HEAD']
+        gitargs = ['symbolic-ref', '-q', 'HEAD']
         ecode, out = b4.git_run_command(gitdir, gitargs)
         if ecode > 0:
-            logger.critical('Not able to get current branch (git branch --show-current)')
+            logger.critical('Not able to get current branch (git symbolic-ref HEAD)')
             sys.exit(1)
-        wantbranch = out.strip()
+        wantbranch = re.sub(r'^refs/heads/', '', out.strip())
         logger.debug('will check branch=%s', wantbranch)
     else:
         # Make sure it's a real branch
-        gitargs = ['branch', '--format=%(refname:short)', '--list', '--all']
+        gitargs = ['branch', '--format=%(refname)', '--list', '--all']
         lines = b4.git_get_command_lines(gitdir, gitargs)
         if not len(lines):
             logger.critical('Not able to get a list of branches (git branch --list --all)')
             sys.exit(1)
-        if cmdargs.branch not in lines:
+        if f'refs/heads/{cmdargs.branch}' not in lines:
             logger.critical('Requested branch %s not found in git branch --list --all', cmdargs.branch)
             sys.exit(1)
         wantbranch = cmdargs.branch
@@ -560,32 +587,35 @@ def get_branch_info(gitdir, branch):
 
     BRANCH_INFO = dict()
 
-    if branch.find('/') < 0:
-        # Not a remote branch
-        return BRANCH_INFO
+    remotecfg = b4.get_config_from_git('branch\\.%s\\.*' % branch)
+    if remotecfg is None or 'remote' not in remotecfg:
+        # Did not find a matching branch entry, so look at remotes
+        gitargs = ['remote', 'show']
+        lines = b4.git_get_command_lines(gitdir, gitargs)
+        if not len(lines):
+            # No remotes? Hmm...
+            return BRANCH_INFO
 
-    # Get a list of all remotes
-    gitargs = ['remote', 'show']
-    lines = b4.git_get_command_lines(gitdir, gitargs)
-    if not len(lines):
-        # No remotes? Hmm...
-        return BRANCH_INFO
+        remote = None
+        for entry in lines:
+            if branch.find(f'{entry}/') == 0:
+                remote = entry
+                break
 
-    remote = None
-    for entry in lines:
-        if branch.find('%s/' % entry) == 0:
-            remote = entry
-            break
+        if remote is None:
+            # Not found any matching remotes
+            return BRANCH_INFO
 
-    if remote is None:
-        # Not found any matching remotes
-        return BRANCH_INFO
+        BRANCH_INFO['remote'] = remote
+        BRANCH_INFO['branch'] = branch.replace(f'{remote}/', '')
 
-    BRANCH_INFO['remote'] = remote
-    BRANCH_INFO['branch'] = branch.replace('%s/' % remote, '')
+    else:
+        BRANCH_INFO['remote'] = remotecfg['remote']
+        if 'merge' in remotecfg:
+            BRANCH_INFO['branch'] = re.sub(r'^refs/heads/', '', remotecfg['merge'])
 
     # Grab template overrides
-    remotecfg = b4.get_config_from_git('remote\\.%s\\..*' % remote)
+    remotecfg = b4.get_config_from_git('remote\\.%s\\..*' % BRANCH_INFO['remote'])
     BRANCH_INFO.update(remotecfg)
 
     return BRANCH_INFO
@@ -612,5 +642,5 @@ def main(cmdargs):
             sys.exit(0)
         write_tracked(tracked)
         logger.info('---')
-        logger.info('You can send them using:')
-        logger.info('  b4 ty -s 1 [2 3 ...]')
+        logger.info('You can send them using number ranges, e.g:')
+        logger.info('  b4 ty -s 1-3,5,7-')
